@@ -16,6 +16,8 @@ public:
     Kernel (const double supportMin, const double supportMax)
         : supportMin(supportMin), supportMax(supportMax) {}
     
+    virtual ~Kernel () {}
+    
     virtual double evaluate (const double x) const { return 0.0; }
     
     double getSupportMin () const { return supportMin; }
@@ -61,22 +63,47 @@ public:
     Array<double> * getArray () const { return values; }
 };
 
-// General polynomial kernel
-// Evaluates to a polynomial function of location, within the support region
-class PolynomialKernel : public Kernel
+template <int N>
+struct PolynomialEvaluator
 {
 private:
-    double term (const double x, const int i) const;
-    
+    const Rcpp::NumericVector *coefficients;
+    const PolynomialEvaluator<N-1> child;
+
+public:
+    PolynomialEvaluator (const Rcpp::NumericVector *coefficients)
+        : coefficients(coefficients), child(coefficients) {}
+
+    double operator() (const double x) const { return (*coefficients)[N] + x * child(x); }
+};
+
+template <>
+struct PolynomialEvaluator<0>
+{
+private:
+    const Rcpp::NumericVector *coefficients;
+
+public:
+    PolynomialEvaluator (const Rcpp::NumericVector *coefficients)
+        : coefficients(coefficients) {}
+
+    double operator() (const double x) const { return (*coefficients)[0]; }
+};
+
+// General polynomial kernel
+// Evaluates to a polynomial function of location, within the support region
+template <int Degree>
+class PolynomialKernel : public Kernel
+{
 protected:
-    int degree;
-    Eigen::VectorXd coefficients;
+    Rcpp::NumericVector coefficients;
+    PolynomialEvaluator<Degree> evaluator;
     
 public:
-    PolynomialKernel (const Eigen::VectorXd &coefficients, const double supportMin, const double supportMax)
-        : Kernel(supportMin,supportMax), coefficients(coefficients)
+    PolynomialKernel (const Rcpp::NumericVector &coefficients, const double supportMin, const double supportMax)
+        : Kernel(supportMin,supportMax), coefficients(coefficients), evaluator(&this->coefficients)
     {
-        this->degree = coefficients.size() - 1;
+        std::reverse(this->coefficients.begin(), this->coefficients.end());
     }
     
     double evaluate (const double x) const;
@@ -119,8 +146,8 @@ public:
 class KernelGenerator
 {
 public:
-    static PolynomialKernel * box ();
-    static PolynomialKernel * triangle ();
+    static PolynomialKernel<0> * box ();
+    static PolynomialKernel<1> * triangle ();
     static CompositeKernel * mitchellNetravali (const double B, const double C);
 };
 
